;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x4f910b7402491356db46d20290a4ba37 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3c36adb6aa6cc28d3b4b468ff66775ba . "./code.cg2js/cg-gesture-emulation.fasl")
(#xf06985c3fbd6d714dcbc41120beed906 . "./code.cg2js/cg-grid-widget.fasl")
(#x70beed963accfa48e7395de2b38a0f02 . "./code.cg2js/cg-utility-dialog.fasl")
(#xe4f04ccca2c3b41dcbc1265f7233d282 . "./code.cg2js/cg-wrap-string.fasl")
(#x9280e12125db685cd5fb6818ce31cd4b . "./code.cg2js/cg-editable-text.fasl")
(#xf1d491d6d5aa3afcf26b9e0c5889da0d . "./code.cg2js/cg-base.fasl")
(#xc909f898632ed30e091aa38c9c5baf43 . "./code.cg2js/cg-lisp-text.fasl")
(#xf0af7ff457ffc0dc862e75a4de907551 . "./code.cg2js/cg-intersect.fasl")
(#x03bfab1b8b93e2940f41fb3f5afab857 . "./code.cg2js/cg-choose-printer.fasl")
(#x3f88c8cb8d52c7fb2cd43fc609f20cde . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x2c7f12c72f49f8a95cee9fbaab10bcc7 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xa20c548d9a78d9128dee79b4ecea1c0b . "./code.cg2js/cg-font-dialog.fasl")
(#x9539f9d66aa414db7c8227582d30d3ef . "./code.cg2js/cg-comtab.fasl")
(#x58a9301e008cc1784569b9ccfc846c9f . "./code.cg2js/cg-tooltip.fasl")
(#x1fddaad0804fe47a6b8e392da1b9cadf . "./code.cg2js/cg-chart-or-plot.fasl")
(#xd888cdc2cdb870329f9e87160190b017 . "./code.cg2js/cg-get-pixmap.fasl")
(#x22a68473fcd7f8b91c5130d25ca37d54 . "./code.cg2js/cg-status-bar.fasl")
(#xdd6f4f158571ebecfc55199b4d0474a8 . "./code.cg2js/cg-tab-control.fasl")
(#x82227b6ed21791cdf54e8c923b6f0d86 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x915a4231f48f664a63455283f60ac7ea . "./code.cg2js/cg-list-view.fasl")
(#x3d7b094cc23d86ecf25fb55c76832009 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x91bf67261590f2995ad9fc2d0ab54089 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9bedfb35bc01ef32fa74ea1d1ef60ab3 . "./code.cg2js/cg-project-window.fasl")
(#x1fab30d20c325f6dabf4674f7485e6b7 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xeabca2121db8bf2a45d913c728cdd6a4 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xcc9c87d6f8b85702a9b4410735ad0d98 . "./code.cg2js/cg-scaling-stream.fasl")
(#x0ffa853cd035fa919acfba301b9f171d . "./code.cg2js/cg-bitmap-stream.fasl")
(#x8aa6350329c9140af3be25805821ac0c . "./code.cg2js/cg-menu.fasl")
(#x1e7909a1eb0e2e9190f99bd10568ee70 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xb7e228d1e10fe9658a43609e8a707398 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xe6c44bf567c9641032f6dded7afb7c23 . "./code.cg2js/cg-split-bar.fasl")
(#xa01744d56260a54934bd7c10b4ce2c03 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x5cb2faf7706ea991953977b9de045516 . "./code.cg2js/cg-timer.fasl")
(#x181c69163d7032f147ebcae041ff4f43 . "./code.cg2js/cg-text-or-combo.fasl")
(#xa37aa3b067f1f79da1675aa10a975230 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x7a5bba471840e07cb2277b37ef7f360e . "./code.cg2js/cg-string-dialog.fasl")
(#xa94da2a4daf332ea6f3cab7a61a18156 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc90b266131876b96286e6021e082432c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x13ac008bc3ef6062f898ea5fb6962edc . "./code.cg2js/cg-outline.fasl")
(#x43afcf807e057c0d4bb864ba8d7a465f . "./code.cg2js/cg-lettered-menu.fasl")
(#xccaf092f2cad7fce0c9e12711438bf8b . "./code.cg2js/cg-graphics-context.fasl")
(#x0b1843451a77f0a0a04a3c708ff14de4 . "./code.cg2js/cg-item-list.fasl")
(#x4cfa522f4d1b23d4bd0f74f4a941648b . "./code.cg2js/cg-caret.fasl")
(#x88ef04f6c138d64b6938ae7c76ae45ee . "./code.cg2js/cg-printing.fasl")
(#x7630d13f77aa3bcf34bb7f23cd467221 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xc8bdecb1e81d2683cffe3c3b09411d95 . "./code.cg2js/cg-progress-indicator.fasl")
(#x6598582e2d2ad80da62ced8196a06da4 . "./code.cg2js/cg-dde-utils.fasl")
(#xeb40997b6b10dd0441b3704bc6c77595 . "./code.cg2js/cg-plot-widget.fasl")
(#x46e2dc3908579f0c68e7f66aab7cf13e . "./code.cg2js/cg-os-window.fasl")
(#x396039502c34263fc67e54b0af40139d . "./code.cg2js/cg-calendar.fasl")
(#x28c661f3d759f56b11030549d1dd0d9a . "./code.cg2js/cg-chart-widget.fasl")
(#x00ee034a6c425f58cf02ad977f5691df . "./code.cg2js/cg-file-dialog.fasl")
(#x109c770d7cade5333fe820d1c39a81d9 . "./code.cg2js/cg-nodes.fasl")
(#x59fc7a9cf7c94a44e11301d336a3290a . "./code.cg2js/cg-lamp.fasl")
(#x9d0aae088a97dca2e7f0d5b39b2654c2 . "./code.cg2js/cg-html-dialog.fasl")
(#xac7325d10ec5f312770762ac90f2ac20 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x8d5c1fd5c2f560511dace8b2842f2fbe . "./code.cg2js/cg-class-slot-grid.fasl")
(#xe570589a53ad8c6cf6033b879c4faf57 . "./code.cg2js/cg-choice-list.fasl")
(#xb2aaf96217242e7ba1c0de0bf1d0625c . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0ae9d809e3ec410e1c5f90655c8220ab . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xa2c6f8f75e49c2d705b578d0c0a016e0 . "./code.cg2js/cg-hotspot.fasl")
(#xf8e6bd8efdef7d9a02a491e68d7ea27b . "./code.cg2js/cg-property.fasl")
(#x36a32ce874decb7b41b2cf27d58cac27 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb7e241836cb2208b94b55885d8df407b . "./code.cg2js/cg-template-string.fasl")
(#x9c34d632f9137550f026767280854b1a . "./code.cg2js/cg-palette.fasl")
(#xa391b21629b3c825c6341362bd289259 . "./code.cg2js/cg-selected-object.fasl")
(#xf274924d46af675347924bf560971941 . "./code.cg2js/cg-up-down-control.fasl")
(#x5d2defc5b497e21ccaf376554da3d961 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x70443d88a338f7008bd94e69f7c13853 . "./code.cg2js/cg-layout.fasl")
(#x5e46ad6f31dd898a6ba0fdbf8b0b3501 . "./code.cg2js/cg-get-position.fasl")
(#x9e3de3e62512044743d038493e716fa6 . "./code.cg2js/cg-lisp-widget.fasl")
(#xcad4ac9ce58c9faff15a02623de88a4e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa20c9649da5c8c1e1e5340c69c6941d8 . "./code.cg2js/cg-object-editor.fasl")
(#x4c10784ccdcca3cc0f47bae2ed3e820d . "./code.cg2js/cg-class-grid.fasl")
(#x4669c97f5ce542dff60ff0d93ce0b5bf . "./code.cg2js/cg-yes-no-string.fasl")
(#x362d1c8708021b5d8af72fadcdc6e166 . "./code.cg2js/cg-toggling-widget.fasl")
(#x2d1ba0a22bfa6c7f158b3d631c2f6ab5 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x960a56c803cbcabbca1240cae0b1413f . "./code.cg2js/cg-button.fasl")
(#x105c86ade4e42723aad01d4c0264564a . "./code.cg2js/cg-object-editor-layout.fasl")
(#xd9889ed9f6579d5daec33386f65362e5 . "./code.cg2js/cg-scroll-bar.fasl")
(#xdbe3e670af14a73e60f0e06e4972313b . "./code.cg2js/cg-combo-box.fasl")
(#xc20bee1f6907ce2f406d9d03bae05b3a . "./code.cg2js/cg-web-browser.fasl")
(#xd0890a8ee437bf84ddac666b8f80470f . "./code.cg2js/cg-drawable.fasl")
(#x821d18c64c833a54ed853bb2795273c2 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x6314f3720bc96d5b6edb59a14c1f4398 . "./code.cg2js/cg-message-dialog.fasl")
(#xdda8ae42aacb32d60df6792fa5b880b4 . "./code.cg2js/cg-text-widget.fasl")
(#x18e3226453df63085fbd50d8a45f2321 . "./code.cg2js/cg-static-text.fasl")
(#x083f76970b9d22310d72f625d7124ba1 . "./code.cg2js/cg-class-support.fasl")
(#x4a0cb56814bd6931e4d55fb2a770307a . "./code.cg2js/cg.fasl")
(#x05ccd39aa92f4b856383ba1f8cd0087f . "./code.cg2js/cg-directory-dialog.fasl")
(#xd960e42bebdba3d8d92f2c14b96d0b3c . "./code.cg2js/cg-clipboard.fasl")
(#x1e1ed133dc5362be1c537239e5e8407b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x037d3082afde362e95ebce3320352ed5 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x08aa9f7c8cad2f192cc29e6945a786da . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x3cf524ac338793bc5c44a42e672b0bd6 . "./code.cg2js/cg-check-box.fasl")
(#x551daf8f381da1423b2590e9fbd3f348 . "./code.cg2js/cg-color-dialog.fasl")
(#xf39d1061a6d9e50c9a03ea8b2da7703e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x8c84d4ecc40bc7dfa0ffbde0945b4f85 . "./code.cg2js/cg-dialog-item.fasl")
(#x5d3f8c1c4fa06e543572a4233f33b7e2 . "./code.cg2js/cg-edit-in-place.fasl")
(#x9ac2ba4426dd9b8c05148a9c538e8f20 . "./code.cg2js/cg-tray.fasl")
(#x6f8edb17ccb758155046288997b08b6c . "./code.cg2js/cg-paren-matching.fasl")
(#x1a9d0e941c42e8073a74d926eb979d23 . "./code.cg2js/cg-player.fasl")
(#x97bb245b7ef8a79cdf73653025f50341 . "./code.cg2js/cg-yes-no-list.fasl")
(#xf140eca06f05bb4a9216a62be62e5b05 . "./code.cg2js/cg-common-control.fasl")
(#x372d67347a5b6a1119ffe88625e95fe4 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x219ddf6ecd49b4b9c931d8c509155344 . "./code.cg2js/cg-os-widget.fasl")
(#x3800d24230ac846f7b3fcc2e210789d0 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x305e0ca3e883bd4d264c01f76eb6d9cb . "./code.cg2js/cg-trackbar.fasl")
(#xc5d5d43896aa849cd7a9a9dbf193ca27 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x01495c0cebda41bc9fb60eee48403ee0 . "./code.cg2js/cg-html-widget.fasl")
(#xedc83ece159d5eead442133c93406a6f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x014c4518ab3f00ce62c32e7fa262b4ff . "./code.cg2js/cg-acache.fasl")
(#xc57de779135e78af6dbbeec6a6ce00db . "./code.cg2js/cg-shortcut-menu.fasl")
(#xa8332e6688d5278ef425d01ebbc0498a . "./code.cg2js/cg-pixmap-widget.fasl")
(#x6bb19900c29ae5399403b1e0ad61a09b . "./code.cg2js/cg-icon.fasl")
(#x6c02a50765c68a2c1f524db72f9211ed . "./code.cg2js/cg-pixmap.fasl")
(#xeb49946492e4dd288bddffb8fab92b62 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x117fa5a9295eb0dda62af33fb3b6dd7a . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x89236b97ba1ad02095d7e8a68998e6de . "./code.cg2js/cg-toolbar.fasl")
(#x0c5c78092022b533a8f82b80dd31aef3 . "./code.cg2js/cg-radio-button.fasl")
(#x45dffead7558ecaec88ef996341b0bd0 . "./code.cg2js/cg-curve.fasl")
(#x2220b053ac77c541b0543ead3e0a4960 . "./code.cg2js/cg-picture-widget.fasl")
(#x1615a1e6088109d78d558a1ddd51815a . "./code.cg2js/cg-dev.fasl")
(#xe80a0b73a252ff2c8b8d86e248afbbfd . "./code.cg2js/cg-group-box.fasl")
(#x4ccc181669c3189f9a8acd1a44f4ee6f . "./code.cg2js/cg-fill-texture.fasl")
(#x9bd61e55d5091353da4900fdbe0f749e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa9f38228c16e24b7ff5c493985f5af7f . "./code.idejs/ide-keyshort.fasl")
(#x27f7531bfd93a2d03f5d8bf870d3c8ad . "./code.idejs/ide-form.fasl")
(#x15c93785c0ea28bab7a74e9e060771a4 . "./code.idejs/ide-find-in-files.fasl")
(#xb37eb372470b8ba38814ab8e48bb4e69 . "./code.idejs/ide-class-grapher.fasl")
(#x84f98a35dfed75d6848a250d3a880447 . "./code.idejs/ide-base.fasl")
(#x8a965acddab41a841af8c173c4ed4239 . "./code.idejs/ide-class-browser.fasl")
(#xfe280f53c5e5cf69e734e846eb8160d9 . "./code.idejs/ide-find-definitions.fasl")
(#xa7a58d82b4d3e5818e762183cd3c2453 . "./code.idejs/ide-package-list.fasl")
(#x04087fc0d89b1668270499eb49ebbb20 . "./code.idejs/ide-grapher.fasl")
(#x34d76cee463389b28f61ea5d4320c4d1 . "./code.idejs/ide-dev.fasl")
(#x6a0b0fdcb40351cadb9d2d780f1a2980 . "./code.idejs/ide.fasl")
(#xfc95881a8f7c5372498361d1f08439b4 . "./code.idejs/ide-trace.fasl")
(#x9d5f8f61a274abbc2b4a12cc364de3c6 . "./code.idejs/ide-apropos.fasl")
(#xbd9cd8b5dbd9a13f0752a01e3b350cbb . "./code.idejs/ide-inspector.fasl")
(#xaddc7dc566baf44904b75b8e609b7a29 . "./code.idejs/ide-acache.fasl")
(#x681dda016857895793541afd8fdc8689 . "./code.idejs/ide-project.fasl")
(#xd75c852e49a9dd967e3c2070e8a4aec1 . "./code.idejs/ide-window-list.fasl")
(#x6359165613932cc8f637a1cc5d6f7fbf . "./code.idejs/ide-intro.fasl")
(#x3933960d7b52a8161443b7472a3a9e07 . "./code.idejs/ide-clipboard.fasl")
(#x65abb3741003b5153051a49ca2a38c2b . "./code.idejs/ide-editor.fasl")
(#x1be83cced441c1906626d01e9a23d5ad . "./code.idejs/ide-process-browser.fasl")
(#x235e74a5d0b5ed69d29249866b138e5d . "./code.idejs/ide-patch-dialog.fasl")
(#x3350beabe6454af2adefcc7e0c2f276a . "./code.idejs/ide-navigator.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
